@@ -1,25 +1,37 @@
-/* 
- * Copyright (C) 1998 - 2011 The GNOME Foundation.
+/*
+ * Copyright (C) 2001 - 2004 Rodrigo Moya <rodrigo@gnome-db.org>
+ * Copyright (C) 2002 - 2003 Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
+ * Copyright (C) 2002 Holger Thon <holger.thon@gnome-db.org>
+ * Copyright (C) 2003 Laurent Sansonetti <lrz@src.gnome.org>
+ * Copyright (C) 2003 Paisa Seeluangsawat <paisa@users.sf.net>
+ * Copyright (C) 2003 Philippe CHARLIER <p.charlier@chello.be>
+ * Copyright (C) 2004 Dani Baeyens <daniel.baeyens@hispalinux.es>
+ * Copyright (C) 2004 Szalai Ferenc <szferi@einstein.ki.iif.hu>
+ * Copyright (C) 2004 - 2005 lvaro Pea <alvaropg@telefonica.net>
+ * Copyright (C) 2005 - 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2005 Dan Winship <danw@src.gnome.org>
+ * Copyright (C) 2005 Stanislav Brabec <sbrabec@suse.de>
+ * Copyright (C) 2005 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2006 - 2010 Murray Cumming <murrayc@murrayc.com>
+ * Copyright (C) 2007 Leonardo Boshell <lb@kmc.com.co>
+ * Copyright (C) 2008 Phil Longstaff <plongstaff@rogers.com>
+ * Copyright (C) 2008 Przemysław Grzegorczyk <pgrzegorczyk@gmail.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *	Rodrigo Moya <rodrigo@gnome-db.org>
- *	Gonzalo Paniagua Javier <gonzalo@gnome-db.org>
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <glib/gi18n-lib.h>
@@ -61,6 +73,7 @@ enum {
 	ROW_UPDATED,
 	ROW_REMOVED,
 	RESET,
+	ACCESS_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -186,6 +199,22 @@ gda_data_model_class_init (G_GNUC_UNUSED gpointer g_class)
 				      g_cclosure_marshal_VOID__VOID,
 				      G_TYPE_NONE, 0);
 
+		/**
+		 * GdaDataModel::access-changed
+		 * @model: the #GdaDataModel
+		 *
+		 * Gets emitted when @model's access flags have changed. Use
+		 * gda_data_model_get_access_flags() to get the access flags.
+		 */
+		gda_data_model_signals[ACCESS_CHANGED] =
+			g_signal_new ("access-changed",
+				      GDA_TYPE_DATA_MODEL,
+				      G_SIGNAL_RUN_LAST,
+				      G_STRUCT_OFFSET (GdaDataModelIface, access_changed),
+				      NULL, NULL,
+				      g_cclosure_marshal_VOID__VOID,
+				      G_TYPE_NONE, 0);
+
 		initialized = TRUE;
 	}
 	g_static_rec_mutex_unlock (&init_mutex);
@@ -699,8 +728,14 @@ gda_data_model_get_attributes_at (GdaDataModel *model, gint col, gint row)
 
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_get_attributes_at)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_get_attributes_at) (model, col, row);
-	else 
-		return 0;
+	else {
+		GdaDataModelAccessFlags flags;
+		GdaValueAttribute attrs = GDA_VALUE_ATTR_NO_MODIF;
+		flags = gda_data_model_get_access_flags (model);
+		if (flags & GDA_DATA_MODEL_ACCESS_WRITE)
+			attrs = 0;
+		return attrs;
+	}
 }
 
 /**
@@ -737,7 +772,7 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
  * gda_data_model_set_values:
  * @model: a #GdaDataModel object.
  * @row: row number.
- * @values: (element-type GObject.Value) (transfer none): a list of #GValue, one for at most the number of columns of @model
+ * @values: (element-type GObject.Value) (transfer none) (allow-none): a list of #GValue (or %NULL), one for at most the number of columns of @model
  * @error: a place to store errors, or %NULL
  *
  * In a similar way to gda_data_model_set_value_at(), this method modifies a data model's contents
@@ -749,15 +784,38 @@ gda_data_model_set_value_at (GdaDataModel *model, gint col, gint row, const GVal
  * Upon errors FALSE will be returned and @error will be assigned a
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
  *
- * Returns: TRUE if the value in the data model has been updated and no error occurred
+ * Returns: %TRUE if the value in the data model has been updated and no error occurred
  */
 gboolean
 gda_data_model_set_values (GdaDataModel *model, gint row, GList *values, GError **error)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL (model), FALSE);
+	
 
 	if (GDA_DATA_MODEL_GET_CLASS (model)->i_set_values)
 		return (GDA_DATA_MODEL_GET_CLASS (model)->i_set_values) (model, row, values, error);
+	else if (GDA_DATA_MODEL_GET_CLASS (model)->i_set_value_at) {
+		/* save the values */
+		gint col, ncols;
+		ncols = gda_data_model_get_n_columns (model);
+		if ((gint)g_list_length (values) > ncols) {
+			g_set_error (error, GDA_DATA_MODEL_ERROR,  GDA_DATA_MODEL_VALUES_LIST_ERROR,
+				     "%s", _("Too many values in list"));
+			return FALSE;
+		}
+
+		for (col = 0;
+		     (col < ncols) && values;
+		     col++, values = values->next) {
+			const GValue *cvalue;
+			cvalue = (const GValue*) values->data;
+			if (!cvalue)
+				continue;
+			if (! gda_data_model_set_value_at (model, col, row, cvalue, error))
+				return FALSE;
+		}
+		return TRUE;
+	}
 	else {
 		/* method not supported */
 		g_set_error (error, GDA_DATA_MODEL_ERROR, GDA_DATA_MODEL_FEATURE_NON_SUPPORTED_ERROR,
@@ -809,13 +867,13 @@ gda_data_model_create_iter (GdaDataModel *model)
 /**
  * gda_data_model_append_values:
  * @model: a #GdaDataModel object.
- * @values: (element-type GObject.Value): #GList of #GValue* representing the row to add.  The
+ * @values: (element-type GObject.Value) (allow-none): #GList of #GValue* representing the row to add.  The
  *          length must match model's column count.  These #GValue
  *	    are value-copied (the user is still responsible for freeing them).
  * @error: a place to store errors, or %NULL
  *
  * Appends a row to the given data model. If any value in @values is actually %NULL, then 
- * it is considered as a default value.
+ * it is considered as a default value. If @values is %NULL then all values are set to their default value.
  *
  * Upon errors -1 will be returned and @error will be assigned a
  * #GError from the #GDA_DATA_MODEL_ERROR domain.
@@ -982,7 +1040,7 @@ gda_data_model_get_row_from_values (GdaDataModel *model, GSList *values, gint *c
  * gda_data_model_send_hint:
  * @model: a #GdaDataModel
  * @hint: (transfer none): a hint to send to the model
- * @hint_value: an optional value to specify the hint, or %NULL
+ * @hint_value: (allow-none): an optional value to specify the hint, or %NULL
  *
  * Sends a hint to the data model. The hint may or may not be handled by the data
  * model, depending on its implementation
@@ -1030,9 +1088,9 @@ static gchar *export_to_text_separated (GdaDataModel *model, const gint *cols, g
  * gda_data_model_export_to_string:
  * @model: a #GdaDataModel
  * @format: the format in which to export data
- * @cols: (array length=nb_cols): an array containing which columns of @model will be exported, or %NULL for all columns
+ * @cols: (array length=nb_cols) (allow-none): an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
- * @rows: (array length=nb_rows): an array containing which rows of @model will be exported, or %NULL for all rows
+ * @rows: (array length=nb_rows) (allow-none): an array containing which rows of @model will be exported, or %NULL for all rows
  * @nb_rows: the number of rows in @rows
  * @options: list of options for the export
  *
@@ -1237,9 +1295,9 @@ gda_data_model_export_to_string (GdaDataModel *model, GdaDataModelIOFormat forma
  * @model: a #GdaDataModel
  * @format: the format in which to export data
  * @file: the filename to export to
- * @cols: (array length=nb_cols): an array containing which columns of @model will be exported, or %NULL for all columns
+ * @cols: (array length=nb_cols) (allow-none): an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
- * @rows: (array length=nb_rows): an array containing which rows of @model will be exported, or %NULL for all rows
+ * @rows: (array length=nb_rows) (allow-none): an array containing which rows of @model will be exported, or %NULL for all rows
  * @nb_rows: the number of rows in @rows
  * @options: list of options for the export
  * @error: a place to store errors, or %NULL
@@ -1391,18 +1449,30 @@ export_to_text_separated (GdaDataModel *model, const gint *cols, gint nb_cols,
 				txt = g_strdup ("");
 			else {
 				gchar *tmp;
-				gsize len, size;
 				
 				tmp = gda_value_stringify (value);
-				len = strlen (tmp);
-				size = 2 * len + 3;
-				txt = g_new (gchar, size);
+				if (tmp) {
+					gsize len, size;
+					len = strlen (tmp);
+					size = 2 * len + 3;
+					txt = g_new (gchar, size);
 
-				len = csv_write2 (txt, size, tmp, len, quote);
-				txt [len] = 0;
-				if (!field_quotes) {
-					txt [len - 1] = 0;
-					memmove (txt, txt+1, len);
+					len = csv_write2 (txt, size, tmp, len, quote);
+					txt [len] = 0;
+					if (!field_quotes) {
+						txt [len - 1] = 0;
+						memmove (txt, txt+1, len);
+					}
+				}
+				else {
+					if (field_quotes) {
+						txt = g_new (gchar, 3);
+						txt [0] = quote;
+						txt [1] = quote;
+						txt [2] = 0;
+					}
+					else
+						txt = g_strdup ("");
 				}
 			}
 			if (c > 0)
@@ -1426,11 +1496,11 @@ xml_set_boolean (xmlNodePtr node, const gchar *name, gboolean value)
 /*
  * gda_data_model_to_xml_node
  * @model: a #GdaDataModel object.
- * @cols: an array containing which columns of @model will be exported, or %NULL for all columns
+ * @cols: (allow-none) (array length=nb_cols): an array containing which columns of @model will be exported, or %NULL for all columns
  * @nb_cols: the number of columns in @cols
- * @rows: an array containing which rows of @model will be exported, or %NULL for all rows
+ * @rows: (allow-none) (array length=nb_rows): an array containing which rows of @model will be exported, or %NULL for all rows
  * @nb_rows: the number of rows in @rows
- * @name: name to use for the XML resulting table.
+ * @name: (allow-none): name to use for the XML resulting table or %NULL.
  *
  * Converts a #GdaDataModel into a xmlNodePtr (as used in libxml).
  *
@@ -1718,7 +1788,7 @@ gda_data_model_add_data_from_xml_node (GdaDataModel *model, xmlNodePtr node, GEr
  * @to: the destination #GdaDataModel
  * @from: the source #GdaDataModel
  * @overwrite: TRUE if @to is completely overwritten by @from's data, and FALSE if @from's data is appended to @to
- * @cols_trans: (element-type gint gint): a #GHashTable for columns translating, or %NULL
+ * @cols_trans: (element-type gint gint) (allow-none): a #GHashTable for columns translating, or %NULL
  * @error: a place to store errors, or %NULL
  *
  * Copy the contents of the @from data model to the @to data model. The copy stops as soon as an error
@@ -1984,7 +2054,7 @@ gda_data_model_import_from_model (GdaDataModel *to, GdaDataModel *from,
  * gda_data_model_import_from_string:
  * @model: a #GdaDataModel
  * @string: the string to import data from
- * @cols_trans: (element-type gint gint): a hash table containing which columns of @model will be imported, or %NULL for all columns, see gda_data_model_import_from_model()
+ * @cols_trans: (element-type gint gint) (allow-none): a hash table containing which columns of @model will be imported, or %NULL for all columns, see gda_data_model_import_from_model()
  * @options: list of options for the export
  * @error: a place to store errors, or %NULL
  *
@@ -2019,7 +2089,7 @@ gda_data_model_import_from_string (GdaDataModel *model,
  * gda_data_model_import_from_file:
  * @model: a #GdaDataModel
  * @file: the filename to import from
- * @cols_trans: (element-type gint gint): a #GHashTable for columns translating, or %NULL, see gda_data_model_import_from_model()
+ * @cols_trans: (element-type gint gint) (allow-none): a #GHashTable for columns translating, or %NULL, see gda_data_model_import_from_model()
  * @options: list of options for the export
  * @error: a place to store errors, or %NULL
  *
